<?php defined( 'ABSPATH' ) || exit;
/**
 * Project: rr_divi_childishly
 * File:    class-rr_divi_childishly.php
 * User:    reskator
 * Author:  Philippe Feryn - Reskator
 *
 * @Description
 * @since       1.0.0
 */
if(! class_exists('RR_Divi_Childishly') ) :
	class RR_Divi_Childishly {
		
		/** Private Static ***********************************************************/
		
		/**
		 * Singleton
		 *
		 * @var null Single instance
		 */
		private static $_instance = null;
		
		/** Protected Static *********************************************************/
		
		/**
		 * Path of the plugin main file
		 *
		 * @var null
		 */
		protected static $_plugin_file = null;
		
		
		/**
		 * Plugin info from main file header
		 *
		 * @var array
		 */
		protected static $_plugin_info = null;
		
		/** Private ****************************************************************/
		
		private $is_divi_installed = false;
		private $is_divi_activated = false;
		private $plugin_page;
		private $form_nonce;
		private $form_action;
		private $form_result;
		private $form_rr_name;
		private $form_rr_author;
		private $form_rr_author_url;
		private $form_rr_version = '1.0';
		private $form_rr_color = 'reskator';
		private $form_rr_colors = array(
			'azur'          => 'Azur',
			'girly'         => 'Girly',
			'lilac'         => 'Lilac',
			'lagon'         => 'Lagoon',
			'turtledove'    => 'Turtledove',
			'chestnut'      => 'Chestnut',
			'forest'        => 'Forest',
			'raspberry'     => 'Raspberry',
			'sunny'         => 'Sunny',
			'reskator'      => 'Reskator',
		); // colors
		private $form_rr_divi_comments = "Yes";
		private $form_rr_divi_css = "Yes";
		private $form_rr_divi_svg = false;
		private $form_rr_divi_selfping = false;
		
		private $rr_msg = array(
			'updated' => array(),
			'notice'  => array(),
			'error'   => array(),
		); // msg
		
		private $gd_loaded;
		private $img_stamp_right = 'reskator_ecusson_blc.png';
		private $text_width;
		private $text_size;
		
		
		/**
		 * Get the instance.
		 *
		 * @since 1.0
		 *
		 * @return Divi_Childishly instance
		 */
		public static function get_instance($plugin_file_path) {
			if ( null === self::$_instance ) {
				self::$_instance = new self($plugin_file_path);
			}
			return self::$_instance;
		}
		
		public function __construct( $plugin_file_path ) {
			
			$this->_set_plugin_info( $plugin_file_path );
			
			load_plugin_textdomain( self::$_plugin_info->textdomain, null, basename( dirname( self::$_plugin_file ) ) . self::$_plugin_info->domain_path );
			
			register_activation_hook( self::$_plugin_file, 'RR_Divi_Childishly_Activate::activate' );
			register_deactivation_hook( self::$_plugin_file, 'RR_Divi_Childishly_Deactivate::deactivate' );
			register_uninstall_hook( self::$_plugin_file, 'RR_Divi_Childishly_Uninstall::uninstall' );
			
			add_filter( 'plugin_action_links_'. plugin_basename( self::$_plugin_file ), [$this, 'rr_divi_childishly_plugin_links'] );
			add_filter( 'plugin_row_meta', [$this, 'rr_divi_plugin_meta_links'], 10 ,2);
			
			add_action( 'init', [$this, 'rr_divi_childishly_admin_register_css_js'] );
			add_action( 'admin_menu', [$this, 'rr_divi_childishly_register_plugin_menu'], 999);
			add_action( 'admin_notices', [$this, 'rr_divi_childishly_dashboard_notice'] );
			add_action( 'wp_ajax_' . self::$_plugin_info->slug_name . '_dismiss-dashboard-notice', [$this, 'dismiss_dashboard_notice'] );
			
		}
		
		
		/**
		 * Retrieves the header information from the plugin main file
		 *
		 * @author  Philippe Feryn
		 * @since   1.0.0
		 *
		 * @param $plugin_file_path Path of the plugin main file
		 */
		private function _set_plugin_info($plugin_file_path) {
			self::$_plugin_info = (object) get_file_data( $plugin_file_path, [
				'display_name' => 'Plugin Name',
				'slug_name'    => 'Plugin Name',
				'version'      => 'Version',
				'textdomain'   => 'Text Domain',
				'domain_path'  => 'Domain Path',
				'wp_min'       => 'Requires at least',
				'php_min'      => 'Requires PHP',
			], 'plugin' );
			
			self::$_plugin_file = $plugin_file_path;
			self::$_plugin_info->slug_name      = str_replace( ' ', '_', strtolower( self::$_plugin_info->slug_name ) );
			$this->gd_loaded = $this->_is_gd_loaded();
			$this->form_nonce  = self::$_plugin_info->slug_name . '_nonce';
			
		}
		
		/**
		 * Show relevant plugin notice.
		 *
		 * @return void
		 */
		public function rr_divi_childishly_dashboard_notice() {
			global $pagenow;
			
			$option = $this->_get_plugin_option( 'Welcome' );
			
			if ( "Yes" == $option ) :
				if ( ! ( $pagenow == 'themes.php' && isset( $_GET['page'] ) && $_GET['page'] == self::$_plugin_info->slug_name ) ) :
					$this->plugin_page = admin_url( 'themes.php?page=' . self::$_plugin_info->slug_name );
					// load the notices view
					include_once( dirname( self::$_plugin_file ) . '/admin/template-dashboard-notice.php' );
				endif;
			endif;
		}
		
		/**
		 * Dismiss the welcome plugin notice.
		 *
		 * @return void
		 */
		public function dismiss_dashboard_notice() {
			check_ajax_referer( self::$_plugin_info->slug_name . '-nonce', 'nonce');
			$this->_update_plugin_option('Welcome', 'No');
		}
		
		/**
		 * Adds a Settings link to this plugin to the plugin settings page
		 *
		 * @author  Philippe Feryn
		 * @since   1.0.0
		 *
		 * @param $links Array of the current links for this plugin
		 *
		 * @return mixed Array updated of the links for this plugin
		 */
		public function rr_divi_childishly_plugin_links( $links ) {
			$settings_link = admin_url( 'themes.php?page=rr_divi_childishly' );
			$settings_link = [
				'Settings'=> wp_kses('<a href="'. $settings_link .'" title="'. __( 'Access to the plugin settings', 'rr_divi_childishly' ) .'">'. __( 'Settings', 'rr_divi_childishly' ) .'</a>', [
					'a' => [
						'href' => [],
						'title' => [],
						'target' => [],
						'aria-label' => [],
						'data-title' => [],
					]
				])
			];
			
			$links = array_merge($settings_link, $links);
			
			return $links;
		}
		
		public function rr_divi_plugin_meta_links( $links, $file ) {
			if( strpos(self::$_plugin_file, $file) !== false ) {
				$new_links = [
					'Doc' => wp_kses('<a href="https://reskator.fr/2019/rr-divi-childishly-plugin-gratuit-theme-enfant-pour-divi/" target="_blank">Documentation</a>', [
										'a' => [
											'href' => [],
											'target' => [],
											'aria-label' => [],
											'data-title' => [],
										]
									]
					),
					'Dons' => wp_kses('<a href="https://paypal.me/pools/c/8aWJC1DfVb" target="_blank">Donate</a>', [
										'a' => [
											'href' => [],
											'target' => [],
											'aria-label' => [],
											'data-title' => [],
										]
									]
					),
				];
				$links = array_merge($links, $new_links);
			}
			
			return $links;
		}
		
		public function rr_divi_childishly_admin_register_css_js() {
			$src = dirname( self::$_plugin_file ) .'/admin/css/rr_divi_childishly-style.css';
			wp_register_style('rr_divi_childishly', $this->_path2url( $src ), [], filemtime( $src ) , 'all' );
			$src = dirname( self::$_plugin_file ) .'/admin/js/rr_divi_childishly.js';
			wp_register_script('rr_divi_childishly', $this->_path2url( $src ), [], filemtime( $src ), false );
		}
		
		public function rr_divi_childishly_register_plugin_menu() {
			$hook_page = add_submenu_page('themes.php', self::$_plugin_info->display_name, self::$_plugin_info->display_name, 'install_themes', self::$_plugin_info->slug_name, [$this, 'rr_divi_childishly_admin_panel'] );
			add_action( 'admin_print_styles-' . $hook_page, [$this, 'rr_divi_childishly_enqueue_css_js'] ) ;
			add_action( 'admin_footer-' . $hook_page, [$this, 'rr_divi_childishly_change_admin_footer'] );
		}
		
		public function rr_divi_childishly_enqueue_css_js() {
			wp_enqueue_style('rr_divi_childishly');
			wp_enqueue_script('rr_divi_childishly');
		}
		
		public function rr_divi_childishly_admin_panel() {
			// only for an authorized user
			if ( ! current_user_can('install_themes') ) {
				printf( '<p>' . __( 'Sorry, you are not allowed to access this page. Return to <a href="%s">Home</a>', 'rr_divi_childishly' ) . '</p>', $this->_get_home_url() );
				return;
			}
			
			// Form submitting
			if ( isset( $_POST['submit'] ) ) {
				// Check nonce
				if ( ! isset( $_POST[ $this->form_nonce ] ) || ! wp_verify_nonce( $_POST[ $this->form_nonce ], self::$_plugin_info->slug_name ) ) {
					$msg = __( 'Missing field ‘nonce’ : Action not performed.', 'rr_divi_childishly' );
					$this->_set_msg( $msg, 'error' );
				} elseif ( ! wp_verify_nonce( $_POST[ $this->form_nonce ], self::$_plugin_info->slug_name ) ) {
					$msg = __( 'Invalid nonce specified. Settings NOT saved.', 'rr_divi_childishly' );
					$this->_set_msg( $msg, 'error' );
				}
				else {
					switch ( $_POST['action'] ) {
						case 'donothing':
							break;
						case 'switch_theme':
							switch_theme( 'Divi' );
							$msg = __( 'Divi theme was activated.', 'rr_divi_childishly' );
							$this->_set_msg( $msg, 'notice_rr' );
							break;
						case 'create':
							$this->form_rr_name         = sanitize_text_field( $_POST['form_rr_name'] );
							$this->form_rr_author       = sanitize_text_field( $_POST['form_rr_author'] );
							$this->form_rr_author_url   = sanitize_text_field( $_POST['form_rr_author_url'] );
							$this->form_rr_version      = sanitize_text_field( $_POST['form_rr_version'] );
							$this->form_rr_divi_css     = (isset($_POST['form_rr_divi_css'])) ? sanitize_text_field( $_POST['form_rr_divi_css'] ) : false;
							$this->form_rr_divi_comments = (isset($_POST['form_rr_divi_comments'])) ? sanitize_text_field( $_POST['form_rr_divi_comments'] ) : false;
							$this->form_rr_divi_svg     = (isset($_POST['form_rr_divi_svg'])) ? sanitize_text_field( $_POST['form_rr_divi_svg'] ) : false;
							$this->form_rr_divi_selfping = (isset($_POST['form_rr_divi_selfping'])) ? sanitize_text_field( $_POST['form_rr_divi_selfping'] ) : false;
							
							if ( $this->gd_loaded ) {
								$this->form_rr_color = $_POST['form_rr_color'];
							}
							
							if ( empty( $this->form_rr_name ) || ! preg_match( "/^[-a-z0-9 ]{1,25}$/i", $this->form_rr_name ) ) {
								$msg = __( 'Child theme’s Name is invalid, empty or contains too much characters (25 max).', 'rr_divi_childishly' );
								$this->_set_msg( $msg, 'error' );
							}
							else {
								$this->form_rr_name = filter_var( $this->form_rr_name, FILTER_SANITIZE_STRING );
							}
							
							if ( empty( $this->form_rr_author ) ) {
								$msg = __( 'Author’s Name has not been entered.', 'rr_divi_childishly' );
								$this->_set_msg( $msg, 'warning' );
							}
							else {
								$this->form_rr_author = filter_var( $this->form_rr_author, FILTER_SANITIZE_STRING );
							}
							
							if ( empty ( $this->form_rr_author_url ) ) {
								$this->form_rr_author_url = $this->_get_home_url();
								$msg = __( 'Author’s URL has not been entered.', 'rr_divi_childishly' );
								$this->_set_msg( $msg, 'warning' );
							}
							else {
								if ( ! preg_match( '/^\bhttps?:\/\/.*\b/i', $this->form_rr_author_url ) ) {
									$this->form_rr_author_url = 'http://' . $this->form_rr_author_url;
								}
								$this->form_rr_author_url = filter_var( $this->form_rr_author_url, FILTER_SANITIZE_URL );
								$this->form_rr_author_url = filter_var( $this->form_rr_author_url, FILTER_VALIDATE_URL );
								if ( ! wp_http_validate_url( $this->form_rr_author_url ) ) {
									$msg = __( 'Author’s URL is detected as invalid. It’s not always accurate, but check if it contains invalid characters or does not start by ‘http://’ or ‘https://’', 'rr_divi_childishly' );
									$this->_set_msg( $msg, 'warning' );
								}
							}
							if ( empty ( $this->form_rr_version ) ) {
								$msg = __( 'Child theme’s version has not been entered.', 'rr_divi_childishly' );
								$this->_set_msg( $msg, 'warning' );
							}
							else {
								$version = $this->_version_number( $this->form_rr_version );
								if ( $version ) {
									$this->form_rr_version = $version;
								}
								else {
									$msg = __( 'The version number is invalid… or with too many subversions.', 'rr_divi_childishly' );
									$this->_set_msg( $msg, 'warning' );
								}
							}
							if ( $this->_get_msg( 'error' ) || $this->_get_msg( 'warning' ) ) {
								$msg = __( 'ERROR: Due to a problem with at least one of the fields, the child theme was not created.', 'rr_divi_childishly' );
								$this->_set_msg( $msg, 'error' );
								$this->form_result = TRUE;
								break;
							}
							else {
								$this->theme_stylesheet = sanitize_file_name( $this->form_rr_name ) . '-child';
								$this->theme_dir        = get_theme_root() . '/' . $this->theme_stylesheet;
								
								if ( @mkdir( $this->theme_dir, 0755 ) ) {
									$msg = __( 'The child theme’s directory has been created.', 'rr_divi_childishly' );
									$this->_set_msg( $msg, 'notice_rr' );
								}
								else {
									$msg = sprintf( __( 'Error: The child theme’s directory %s has NOT been created.', 'rr_divi_childishly' ), $this->theme_stylesheet );
									$this->_set_msg( $msg, 'error' );
									$this->_error_end(); //on arrête là et on affiche la page de résultat
									$this->form_result = TRUE;
									break;
								}
								$plugin_dir = dirname(self::$_plugin_file);
								$style_file     = $plugin_dir .'/admin/assets/style.css';
								$functions_file = $plugin_dir . '/admin/assets/functions.php';
								if ( file_exists( $style_file ) && file_exists( $functions_file ) ) {
									$style_content = file_get_contents( $style_file );
									$themeURI       = wp_get_theme();
									$themeURI       = $themeURI->get( 'ThemeURI' );
									//NextV : Ajouter la gestion des Tags, et l'ajout ou non de la license GPLv2
									
									// Header of the child style.css
									$search        = array(
										'%Name%',
										'%ThemeURI%',
										'%Author%',
										'%AuthorURI%',
										'%Description%',
										'%Template%',
										'%Version%',
										'%Tags%',
										'%TextDomain%',
									);
									$replace = array(
										$this->form_rr_name .' child',
										$themeURI,
										$this->form_rr_author,
										$this->form_rr_author_url,
										'Divi child theme',
										'Divi',
										$this->form_rr_version,
										'',
										'divi-child',
									);
									$style_content = str_replace( $search, $replace, $style_content );
									$file          = $this->theme_dir . '/style.css';
									if ( file_put_contents( $file, $style_content ) ) {
										$msg = __( 'The style.css file has been created.', 'rr_divi_childishly' );
										$this->_set_msg( $msg, 'notice_rr' );
									}
									else {
										$msg = __( 'The style.css file has NOT been created.', 'rr_divi_childishly' );
										$this->_set_msg( $msg, 'error' );
										$this->_error_end(); // we stop there and we display the result page
										break;
									}
									
									$functions_content = file_get_contents( $functions_file );
									
									// Add standard CSS or Dyn CSS
									$search = '%rr_css%';
									$replace = '';
									if( $this->form_rr_divi_css ) {
										$replace = file_get_contents($plugin_dir .'/admin/assets/snippet_css_dyn.lib');
									}
									else {
										$replace = file_get_contents( $plugin_dir .'/admin/assets/snippet_css.lib');
									}
									$functions_content = str_replace( $search, $replace, $functions_content );
									
									// Customize the parent and child Names theme in functions.php
									$search = array(
										'%Name%',
										'%parent_style%',
										'%child_style%',
									);
									$replace = array(
										$this->form_rr_name .' child',
										'divi-style',
										strtolower(sanitize_file_name($this->form_rr_name) ),
									);
									$functions_content = str_replace( $search, $replace, $functions_content );
									
									// Maybe add function to remove divi html comments
									$search = '%rr_divi_comments%';
									$replace = '';
									if( $this->form_rr_divi_comments ) {
										$replace = file_get_contents( $plugin_dir .'/admin/assets/snippet_remove_comments.lib');
									}
									$functions_content = str_replace( $search, $replace, $functions_content );
									
									// Maybe add svg support
									$search = '%rr_divi_svg%';
									$replace = '';
									if( $this->form_rr_divi_svg ) {
										$replace = file_get_contents( $plugin_dir .'/admin/assets/snippet_svg_support.lib');
									}
									$functions_content = str_replace( $search, $replace, $functions_content );
									
									// Maybe add disable self pingback
									$search = '%rr_divi_selfping%';
									$replace = '';
									if( $this->form_rr_divi_selfping ) {
										$replace = file_get_contents( $plugin_dir .'/admin/assets/snippet_disable_self_pingback.lib');
									}
									$functions_content = str_replace( $search, $replace, $functions_content );
									
									$file = $this->theme_dir . '/functions.php';
									
									if ( file_put_contents( $file, $functions_content ) ) {
										$msg = __( 'The functions.php file has been created.', 'rr_divi_childishly' );
										$this->_set_msg( $msg, 'notice_rr' );
									}
									else {
										$msg = __( 'The functions.php file has NOT been created.', 'rr_divi_childishly' );
										$this->_set_msg( $msg, 'error' );
										$this->_error_end(); //on arrête là et on affiche la page de résultat
										break;
									}
									// Creating the custom image.
									if ( $this->gd_loaded ) {
										$this->form_rr_color = ( empty ( $this->form_rr_color ) ) ? 'reskator' : $this->form_rr_color;
										$img_dir              = $plugin_dir . '/admin/images/';
										//$img_stamp            = $plugin_dir .'/admin/assets/'. $this->img_stamp_right;
										$img_stamp            = '';
										$img_src              = $img_dir . 'screenshot-' . $this->form_rr_color . '.png';
										$img_dest             = $this->theme_dir . '/screenshot.png';
										$text                 = array(
											$this->form_rr_name . ' : a Divi Child Theme created by ' . $this->form_rr_author, // the text
											24, // the font size
											15, // the x position
											35, // the y position
										);
										if ( $this->_create_childtheme_img( $img_stamp, $img_src, $img_dest, $text ) ) {
											$msg = __( 'The image file has been created.', 'rr_divi_childishly' );
											$this->_set_msg( $msg, 'notice_rr' );
										}
										else {
											$msg = __( 'The image could not be created, but it does not matter: the child theme is operational.', 'rr_divi_childishly' );
											$this->_set_msg( $msg, 'warning' );
										}
										if ( file_exists( $img_dest ) ) {
											if ( $this->_png2jpeg( $img_dest, 99, TRUE ) ) {
												$msg = __( 'The size image has been compressed.', 'rr_divi_childishly' );
												$this->_set_msg( $msg, 'notice_rr' );
											}
										}
									}
								}
								else {
									$msg = __( 'ERROR: It seems that some files are missing. Thank you to reinstall the plugin.' );
									$this->_set_msg( $msg, 'error' );
								}
								
								$this->form_result = TRUE;
								$this->_set_form_action();
								switch_theme( $this->theme_stylesheet );
							}
							break;
						case 'themes':
							$redirect = admin_url() . '/themes.php';
							wp_redirect( $redirect );
							exit;
							break;
						case 'deactivate':
							deactivate_plugins( plugin_basename( self::$_plugin_file ) );
							$redirect = admin_url() . '/themes.php';
							wp_redirect( $redirect );
							exit;
							break;
						default:
					} // end switch
				}
			} // end Form Submit
			
			if ( $this->form_result ) {
				require( dirname(self::$_plugin_file) .'/admin/template-admin-result.php' );
			}
			else {
				//
				$themes = wp_get_themes( array(
					'Name',
					'Version',
					'ThemeURI',
				) );
				foreach ( $themes as $theme ) {
					if ( 'Divi' == $theme['Name'] ) {
						$this->is_divi_installed = TRUE;
					};
				}
				unset( $themes, $theme ); // Frees memory
				
				if( $this->is_divi_installed ) {
					
					$current_theme = wp_get_theme();
					if ( 'Divi' == $current_theme ) {
						$this->is_divi_activated = TRUE;
						$this->_set_form_action( 'create' );
					}
					else {
						$this->_set_form_action( 'switch_theme' );
					}
					
					if ( $this->is_divi_activated ) {
						$current_user = wp_get_current_user();
						if ( ! ( $current_user instanceof WP_User ) ) {
							return;
						}
						
						// Child theme's name field
						$this->form_rr_name = get_bloginfo( 'name' );
						
						// Author field
						if ( ! empty ( $current_user->user_firstname ) && ! empty ( $current_user->user_lastname ) ) {
							$this->form_rr_author = $current_user->user_firstname . ' ' . $current_user->user_lastname;
						}
						elseif ( ! empty ( $current_user->display_name ) ) {
							$this->form_rr_author = $current_user->display_name;
						}
						else {
							$this->form_rr_author = $current_user->user_login;
						}
						
						// Author url field
						$this->form_rr_author_url = $this->_get_home_url();
					}
				}
				
				require( dirname(self::$_plugin_file) .'/admin/template-admin-panel.php' );
			}
			
			
		}
		
		private function _set_form_action( $query = '' ) {
			$this->form_action = self::_get_current_url();
			$dum = explode( '&', $this->form_action );
			$this->form_action = $dum[0];
			
			if ( ! empty( $query ) ) {
				$this->form_action .= '&action='. $query;
			}
			
			$this->form_action = esc_url( $this->form_action );
		}
		
		/**
		 * Sets a message for a given type
		 *
		 * @author  Philippe Feryn
		 * @since   1.0.0
		 *
		 * @callby
		 * @hook
		 *
		 * @param        $msg
		 * @param string $type
		 */
		private function _set_msg( $msg, $type = 'info' ) {
			$this->rr_msg[ $type ][] = $msg;
		}
		
		/**
		 * Returns the messages of a given type
		 *
		 * @author  Philippe Feryn
		 * @since   1.0.0
		 *
		 * @callby
		 * @hook
		 *
		 * @param string $type
		 *
		 * @return string|void
		 */
		private function _get_msg( $type = 'notice_rr' ) {
			$msg    = '';
			
			if( ! isset( $this->rr_msg[ $type ] ) )
				return;
			
			$nb_msg = count( $this->rr_msg[ $type ] );
			if ( $nb_msg ) {
				$msg = ( $type == 'notice_rr' ) ? '<b>'. __('For your information:', 'rr_divi_childishly') .'</b><br/>' : '';
				$i   = 0;
				foreach ( $this->rr_msg[ $type ] as $value ) {
					$i ++;
					$msg .= ( $type == 'notice_rr' ) ? '• ' . $value : $value;
					if ( $i < $nb_msg ) {
						$msg .= '<br/>';
					}
				}
			}
			
			return $msg;
		}
		
		/**
		 * Checks the version number (from form)
		 *
		 * @author  Philippe Feryn
		 * @since   1.0.0
		 *
		 * @callby
		 * @hook
		 *
		 * @param        $string
		 * @param string $sub
		 *
		 * @return bool
		 */
		private function _version_number( $string, $sub = '' ) {
			if ( empty( $string ) ) {
				return false;
			}
			// {0,2} : until 2 subversions => v1.2.3. If need more, use $sub
			$pattern = "/^(?:(\d+)\.){0,2}(\*|\d+)$/";
			if ( ! empty( $sub ) ) {
				$pattern = str_replace( '2', $sub, $pattern );
			}
			
			// Normalize
			$search  = array( ',', '-', '_' );
			$replace = '.';
			$version = str_replace( $search, $replace, $string );
			
			if ( preg_match( $pattern, $version, $matches ) ) {
				return $matches[0];
			}
		}
		
		/**
		 * Replaces the footer for our plugin’s pages
		 *
		 * @author  Philippe Feryn
		 * @since   1.0.0
		 *
		 * @callby
		 * @hook
		 *
		 */
		public function rr_divi_childishly_change_admin_footer() {
			global $wp_version;
			$output = <<<_HTML
	<div id="rrfooter" role="contentinfo">
		<p id="atp-footer-left" class="alignleft">
			<span id="footer-thankyou"> Plugin réalisé par <a href="https://www.reskator.fr" target="_blank">ReskatoR</a> • <a href="https://www.facebook.com/groups/mon.wordpress/" target="_blank">WordPress, Divi, WooCommerce Assistance</a></span>
		</p>
		<p id="atp-footer-upgrade" class="alignright">
			<a href="wpfr.net">WordPress</a> Version $wp_version
		</p>
		<div class="clear"></div>
	</div>
_HTML;
			echo $output;
		}
		
		/**
		 * Updates the value of a plugin option (wp_options)
		 *
		 * @author  Philippe Feryn
		 * @since   1.0.0
		 *
		 * @callby
		 * @hook
		 *
		 * @param $option
		 * @param $new_value
		 */
		private function _update_plugin_option( $option, $new_value ) {
			if( empty( $option ) )
				return;
			
			$plugin_option = get_option( 'RR_Divi_Childishly_plugin' );
			$plugin_option[ $option ] = ( empty( $new_value ) ) ? $plugin_option[ $option ] : $new_value;
			update_option( 'RR_Divi_Childishly_plugin', $plugin_option, true );
			unset($plugin_option);
		}
		
		/**
		 * Retrieves the value of an option from the plugin (wp_options table) and returns it if found
		 *
		 * @author  Philippe Feryn
		 * @since   1.0.0
		 *
		 * @callby
		 * @hook
		 *
		 * @param $option   option to retrieve
		 */
		protected function _get_plugin_option( $option ) {
			if( empty( $option ) )
				return;
			
			$plugin_options = get_option('RR_Divi_Childishly_plugin');
			if( empty( $plugin_options ) )
				return;
			
			if( array_key_exists( $option, $plugin_options ) )
				return $plugin_options[ $option ];
		}
		
		/**
		 * If an error occurs, stop the process and display the result page
		 *
		 * @author  Philippe Feryn
		 * @since   1.0.0
		 *
		 * @callby
		 * @hook
		 *
		 * @param string $action
		 */
		private function _error_end( $action = 'donothing' ) {
			$this->_set_form_action( $action );
			$this->plugin_result = true;
		}
		
		/**
		 *  Returns the current url, with or without arg(s).
		 *
		 * @author  Philippe Feryn
		 * @since   1.0.0
		 *
		 * @callby
		 * @hook
		 *
		 * @static
		 *
		 * @param bool $with_arg
		 *
		 * @return string
		 */
		protected static function _get_current_url( $with_arg = true ) {
			$url = $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
			
			if( true === $with_arg ) {
				$url = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https://' : 'http://' ) . $url;
			}
			if( false === $with_arg ) {
				$url = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https://' : 'http://' ) . parse_url($url, PHP_URL_PATH );
			}
			
			return $url;
		}
		
		/**
		 * Converts path to url and returns the result.
		 *
		 * @author  Philippe Feryn
		 * @since   1.0.0
		 *
		 * @param string $path
		 * @param string $method
		 *
		 * @return string url
		 */
		public function _path2url( $path = '', $method = 'php' ) {
			if( 'php' == strtolower($method) ) {
				$url = str_replace( $_SERVER['DOCUMENT_ROOT'], '', $_SERVER['SERVER_NAME'] . $path );
				$url = (isset( $_SERVER['HTTPS'] ) && 'on' === strtolower( $_SERVER['HTTPS'] ) ? 'https://' : 'http://' ) . $url;
			}
			else {
				$url = site_url() .'/'. str_replace( ABSPATH, '', $path ); // WordPress method
			}
			
			return $url;
		}
		
		private function _get_home_url() {
			return site_url();
		}
		
		/**
		 * Generating the child theme image
		 *
		 * @author  Philippe Feryn
		 * @since   1.0.0
		 *
		 * @callby  func    rr_divi_childishly_admin_panel() if case = create
		 * @hook
		 *
		 * @param string $img_stamp
		 * @param string $img_src
		 * @param string $img_dest
		 * @param array  $text
		 *
		 * @return bool $result 
		 */
		private function _create_childtheme_img( $img_stamp = '', $img_src = '', $img_dest = '', $text = array() ) {
			$result = true;
			if ( empty( $img_dest ) ) {
				$img_dest = dirname( $img_src ) . '/myimage.jpg';
			}
			
			$img = imagecreatetruecolor( 880, 660 ); // create a true color image of 880x660
			$img_temp = $this->_create_ressource_image( $img_src ); // Function to create a ressource image
			
			// Get size image
			$img_width  = imagesx( $img );
			$img_height = imagesy( $img );
			
			imagecopyresampled($img, $img_temp, 0, 0 ,0, 0, $img_width, $img_height, $img_width, $img_height );
			
			if(!empty($img_stamp)) {
				$stamp = $this->_create_ressource_image( $img_stamp ); // Call a function to create a ressource image
				
				$stamp_width  = imagesx( $stamp );
				$stamp_height = imagesy( $stamp );
				
				// dst_image    Lien vers la ressource cible de l'image.
				// src_image    Lien vers la ressource source de l'image.
				// dst_x        X : coordonnées du point de destination.
				// dst_y        Y : coordonnées du point de destination.
				// src_x        X : coordonnées du point source.
				// src_y        Y : coordonnées du point source.
				// dst_w        Largeur de la destination.
				// dst_h        Hauteur de la destination.
				// src_w        Largeur de la source.
				// src_h        Hauteur de la source.
				
				imagecopyresampled( $img, $stamp, $img_width - ( $stamp_width + 15 ), 15, 0, 0, $stamp_width, $stamp_height, $stamp_width, $stamp_height );
				
				// Frees memory
				imagedestroy( $stamp );
			}
			
			// Add text
			list( $text, $this->text_size, $text_pos_x, $text_pos_y ) = $text;
			$font_path = dirname(self::$_plugin_file) .'/admin/assets/CaviarDreams_Bold.ttf';
			
			// Add a shadow to text
			$grey = imagecolorallocate( $img, 127, 127, 127 );
			$this->_gd_check_txt_width($img_width, $font_path, $text, $this->text_size, 0);
			$text_pos_x = (($img_width-12) - $this->text_width )/2; // centrer le texte (10 : padding left).
			$void = imagettftext( $img, $this->text_size, 0, $text_pos_x, $img_height - ($text_pos_y-2), $grey, $font_path, $text );
			
			// Add white text
			$white = imagecolorallocate( $img, 255, 255, 255 );
			$this->_gd_check_txt_width($img_width, $font_path, $text, $this->text_size, 0);
			$text_pos_x = (($img_width-10) - $this->text_width )/2; // centrer le texte (10 : padding left).
			
		
			$void = imagettftext( $img, $this->text_size, 0, $text_pos_x, $img_height - $text_pos_y, $white, $font_path, $text );
			
			// Create and save image
			if ( ! imagepng( $img, $img_dest, 9, null ) ) {
				$result = false;
			};
			
			sleep( 1 ); // Wait 1 second to write the file on disk, before the next action
			
			imagedestroy( $img ); // Frees memory
			
			return $result;
		}
		
		/**
		 * Check the length of the text and if it is too long, decrease the font size
		 *
		 * @author  Philippe Feryn
		 * @since   1.0.0
		 *
		 * @callby  func    _create_childtheme_img()
		 * @hook
		 *
		 * @param   int     $max_width
		 * @param   string  $font_path
		 * @param   string  $text
		 * @param   int     $text_size
		 * @param   int     $angle
		 */
		private function _gd_check_txt_width( $max_width, $font_path, $text, $text_size = 24, $angle = 0 ) {
			$bbox = imagettfbbox($text_size, $angle, $font_path, $text);
			$this->text_width = abs($bbox[4] - $bbox[0] );
			$this->text_size = $text_size;
			
			if($this->text_width > ($max_width - 20) ) { // 20 = padding (10 left, 10 right)
				$this->text_size -= 2;
				$this->_gd_check_txt_width( $max_width, $font_path, $text, $this->text_size, $angle );
			}
		}
		
		/**
		 * Convert a png image to jpeg
		 *
		 * @param      $img_path
		 * @param int  $quality     medium 75, best 99, no compression 100
		 * @param bool $delete_img  suppress the source (png) image ?
		 *
		 * @return bool|null
		 */
		private function _png2jpeg( $img_path, $quality = 75, $delete_img = false ) {
			$result     = null;
			$path_parts = pathinfo( $img_path );
			$img_dest   = $path_parts['dirname'] . '/' . $path_parts['filename'] . '.jpg';
			
			$img    = $this->_create_ressource_image( $img_path );
			$result = ( imagejpeg( $img, $img_dest, $quality ) ) ? true : false;
			
			if ( $delete_img ) {
				unlink( $img_path );
			}
			
			imagedestroy( $img ); // Frees memory
			
			return $result;
		}
		
		/**
		 * creates and returns an image resource
		 *
		 * @param   string  $img_path
		 * @param   int     $width
		 * @param   int     $height
		 *
		 * @return resource
		 */
		private function _create_ressource_image( $img_path, $width = 880, $height = 660 ) {
			$img_info = getimagesize( $img_path );
			
			switch ( $img_info[2] ) {
				case 1 : // gif
					$img = imagecreatefromgif( $img_path );
					break;
				case 2 : // jpeg
					$img = imagecreatefromjpeg( $img_path );
					break;
				case 3 : // png
					$img = imagecreatefrompng( $img_path );
					break;
				case 6 : // bmp
					$img = imagecreatefrombmp( $img_path );
					break;
				default : // Créer une image vierge
					$img = @imagecreatetruecolor( $width, $height )
					or die( 'Impossible de créer un flus d’image GD' );
			}
			
			return $img;
		}
		
		/**
		 * gd library is loaded ?
		 *
		 * @return bool
		 */
		private function _is_gd_loaded() {
			return ( extension_loaded('gd') ) ? true : false;
		}
	}
endif;