<?php defined( 'ABSPATH' ) || exit;
/**
 * Project: rr_divi_childishly
 * File:    class-rr_divi_childishly_autoloader.php
 * User:    reskator
 * Author:  Philippe Feryn - Reskator
 *
 * @Description Class autoloader for the RR Divi Childishly plugin
 * @since       1.0.0
 */

class RR_Divi_Childishly_Autoloader {
	/**
	 * Singleton method
	 *
	 * If it has not already been done, creates and return an instance of the class
	 *
	 * @author Philippe Feryn
	 * @since  1.0.0
	 *
	 * @static
	 * @return \RR_CDPImmo_Autoloader | null
	 */
	public static function get_instance() {
			if ( ! self::$_instance ) {
				self::$_instance = new self();
			}
			
			return self::$_instance;
		}
	
	/**
		 * Singleton
		 *
		 * @static
		 * @var null Single instance
		 */
	private static $_instance = null;
	
	
	private function __construct() {
			spl_autoload_register( [ $this, 'load' ] );
		}
	
	/**
		 * Class loader
		 *
		 * Converts the name of the class to a file name and includes the class file.
		 *
		 * @author Philippe Feryn
		 * @since  1.0.0
		 *
		 * @param string $class_name - class to load
		 */
	public function load( $class_name ) {
		$file = __DIR__ .'/class-'. strtolower( $class_name );
		
		spl_autoload( $file, '.php' );
	}
	
	/**
		 * Make sure an instance can't be cloned
		 *
		 * @author Philippe Feryn
		 * @since  1.0.0
		 *
		 */
	private function __clone() {
			// do nothing
		}
}
RR_Divi_Childishly_Autoloader::get_instance();